;;; guile-irrlicht --- FFI bindings for Irrlicht Engine
;;; Copyright (C) 2020 Javier Sancho <jsf@jsancho.org>
;;;
;;; This file is part of guile-irrlicht.
;;;
;;; Guile-irrlicht is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU Lesser General Public License as
;;; published by the Free Software Foundation; either version 3 of the
;;; License, or (at your option) any later version.
;;;
;;; Guile-irrlicht is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU Lesser General Public
;;; License along with guile-irrlicht.  If not, see
;;; <http://www.gnu.org/licenses/>.


(define-module (irrlicht video)
  #:use-module (oop goops)
  #:use-module (ice-9 optargs)
  #:use-module (irrlicht base)
  #:use-module (irrlicht foreign))


;; ITexture
(define-class <texture> (<irrlicht-base>)
  (irr-class #:init-value "ITexture"))

(export <texture>)


;; IVideoDriver
(define-class <video-driver> (<irrlicht-base>)
  (irr-class #:init-value "IVideoDriver"))

(define-method (begin-scene (video-driver <video-driver>) . rest)
  (let-keywords rest #f
        ((back-buffer #t)
         (z-buffer #t)
         (color '(255 0 0 0))
         video-data
         source-rect)
    ((get-irrlicht-proc "beginScene" video-driver)
     video-driver
     back-buffer
     z-buffer
     color
     video-data
     source-rect)))

(define-method (end-scene (video-driver <video-driver>))
  ((get-irrlicht-proc "endScene" video-driver)
   video-driver))

(define-method (get-fps (video-driver <video-driver>))
  (let ((getFPS (get-irrlicht-proc "getFPS" video-driver)))
    (getFPS video-driver)))

(define-method (get-name (video-driver <video-driver>))
  (let ((getName (get-irrlicht-proc "getName" video-driver)))
    (getName video-driver)))

(define-method (get-texture (video-driver <video-driver>) filename)
  (make <texture>
    #:irr-pointer
    ((get-irrlicht-proc "getTexture" video-driver)
     video-driver
     filename)))

(export <video-driver> begin-scene end-scene get-fps get-name get-texture)


;; S3DVertex
(define-class <vertex3d> (<irrlicht-base>)
  (irr-class #:init-value "S3DVertex"))

(define-method (get-position (vertex3d <vertex3d>))
  (let ((S3DVertex_Pos (get-irrlicht-proc "S3DVertex_Pos")))
    (S3DVertex_Pos vertex3d)))

(define (make-vertex3d position normal color tcoords)
  (let ((S3DVertex_make (get-irrlicht-proc "S3DVertex_make")))
    (make <vertex3d>
      #:irr-pointer
      (S3DVertex_make position normal color tcoords))))

(export <vertex3d> get-position make-vertex3d)


;; SMaterial
(define-class <material> (<irrlicht-base>)
  (irr-class #:init-value "SMaterial"))

(define* (make-material #:key
                        (material-type 'solid)
                        (ambient-color '(255 255 255 255))
                        (diffuse-color '(255 255 255 255))
                        (emissive-color '(0 0 0 0))
                        (specular-color '(255 255 255 255))
                        (shininess 0)
                        (material-type-param 0)
                        (material-type-param-2 0)
                        (thickness 1)
                        (z-buffer 'less-equal)
                        (anti-aliasing 'simple)
                        (color-mask 'all)
                        (color-material 'diffuse)
                        (blend-operation 'none)
                        (polygon-offset-factor 0)
                        (polygon-offset-direction 'front)
                        (wireframe #f)
                        (point-cloud #f)
                        (gouraud-shading #t)
                        (lighting #t)
                        (z-write-enable #t)
                        (backface-culling #t)
                        (frontface-culling #f)
                        (fog-enable #f)
                        (normalize-normals #f)
                        (use-mip-maps #t))
  (let ((SMaterial_make (get-irrlicht-proc "SMaterial_make")))
    (make <material>
      #:irr-pointer
      (SMaterial_make #:material-type material-type #:ambient-color ambient-color
                      #:diffuse-color diffuse-color #:emissive-color emissive-color
                      #:specular-color specular-color #:shininess shininess
                      #:material-type-param material-type-param
                      #:material-type-param-2 material-type-param-2
                      #:thickness thickness #:z-buffer z-buffer #:anti-aliasing anti-aliasing
                      #:color-mask color-mask #:color-material color-material
                      #:blend-operation blend-operation
                      #:polygon-offset-factor polygon-offset-factor
                      #:polygon-offset-direction polygon-offset-direction
                      #:wireframe wireframe #:point-cloud point-cloud
                      #:gouraud-shading gouraud-shading #:lighting lighting
                      #:z-write-enable z-write-enable #:backface-culling backface-culling
                      #:frontface-culling frontface-culling #:fog-enable fog-enable
                      #:normalize-normals normalize-normals #:use-mip-maps use-mip-maps))))

(export <material> make-material)
